@@ -3,18 +3,19 @@ from nltk.sentiment.vader import SentimentIntensityAnalyzer
 from nltk.corpus import stopwords
 from nltk.stem import WordNetLemmatizer
 from nltk.tokenize import sent_tokenize
-import nltk
 from GoogleNews import GoogleNews
 from newspaper import Article, ArticleException
 from tqdm import tqdm, trange
 from threading import Thread
-from datetime import date, datetime, timedelta
+from datetime import datetime, timedelta
 import re
 import pandas as pd
 import time, requests
 import warnings
-import sys, logging
+import logging
 import twint
+from finvizfinance.quote import finvizfinance
+from pytrends.request import TrendReq
 
 from dateutil.parser._parser import UnknownTimezoneWarning
 import flair
@@ -25,15 +26,15 @@ warnings.simplefilter("ignore", UnknownTimezoneWarning)
 
 # define time formats for each
 GOOGLENEWS_TF = "%m/%d/%Y"
-# TWITTER_TF = "%Y-%m-%d"
-TWITTER_TF = "%Y-%m-%dT%H:%M:%SZ"
-TWINT_TF = "%Y-%m-%d %H:%M:%S"
+TRENDS_TF = "%Y-%m-%d"  # google trends
+TWITTER_TF = "%Y-%m-%dT%H:%M:%SZ"  # twitter api
+TWINT_TF = "%Y-%m-%d %H:%M:%S"  # twint api
 
 NEWSAPI_URL = "https://newsapi.org/v2/everything?"
 TWITTER_URL = "https://api.twitter.com/2/tweets/search/recent"
 
 
-class NewsAPI:
+class NewsAPIParser:
     """
     A class used to fetch news from newsapi.org
 
@@ -43,6 +44,8 @@ class NewsAPI:
     ----------
     newsapi_key : str
         api key to connect to newsapi.org
+    tqdisable : bool
+        disale progressbars
 
     Methods
     -------
@@ -76,7 +79,7 @@ class NewsAPI:
         pagesize: int = 100,
         page: int = 1,
         language: str = "en",
-        start_time=(datetime.now() - timedelta(2)),
+        start_time=(datetime.now() - timedelta(days=2)),
         end_time=datetime.now(),
     ):
         """Search the news for a search term.
@@ -93,10 +96,10 @@ class NewsAPI:
             page to read from
         language: str = "en", optional
             language to search in
-        start_time : str = (datetime.now() - timedelta(2))
-            how far back to search from in time format %Y-%m-%dT%H:%M:%SZ'
-        end_time : str = datetime.now()
-            how recent to search from in time format %Y-%m-%dT%H:%M:%SZ'
+        start_time : timedelta = (datetime.now() - timedelta(days=2))
+            how far back to search from
+        end_time : timedelta = datetime.now()
+            how recent to search from
 
         Returns
         -------
@@ -163,7 +166,7 @@ class NewsAPI:
     def get_articles(
         self,
         searchfor,
-        start_time=(datetime.now() - timedelta(2)),
+        start_time=(datetime.now() - timedelta(days=2)),
         end_time=datetime.now(),
     ):
         """Gets articles for a single search term
@@ -205,7 +208,8 @@ class GoogleNewsParser:
 
     Attributes
     ----------
-    None
+    tqdisable : bool
+        disale progressbars
 
     Methods
     -------
@@ -215,7 +219,8 @@ class GoogleNewsParser:
         Gets articles for a single search term
     """
 
-    def __init__(self) -> None:
+    def __init__(self, tqdisable=True) -> None:
+        self.tqdisable = tqdisable
         self.googlenews = GoogleNews()  # create news object
 
     def _get_text(self, inst):
@@ -251,7 +256,7 @@ class GoogleNewsParser:
     def get_articles(
         self,
         search_term,
-        start_time=(datetime.now() - timedelta(2)),
+        start_time=(datetime.now() - timedelta(days=2)),
         end_time=datetime.now(),
     ):
         """Gets articles for a single search term
@@ -260,10 +265,10 @@ class GoogleNewsParser:
         ----------
         searchfor: str
             term to search for
-        start_time : str = (datetime.now() - timedelta(2))
-            how far back to search from in time format %Y-%m-%dT%H:%M:%SZ'
-        end_time : str = datetime.now()
-            how recent to search from in time format %Y-%m-%dT%H:%M:%SZ'
+        start_time : timedelta = (datetime.now() - timedelta(days=2))
+            how far back to search from
+        end_time : timedelta = datetime.now()
+            how recent to search from
 
         Returns
         -------
@@ -308,6 +313,117 @@ class GoogleNewsParser:
         return pd.DataFrame(results)
 
 
+class FinvizParser:
+    """
+    A class used to fetch news from finviz
+
+    ...
+
+    Attributes
+    ----------
+    tqdisable : bool
+        disale progressbars
+
+    Methods
+    -------
+    _get_text(inst)
+        gets the text for each article it recieves
+    get_articles
+        Gets articles for a single search term
+    """
+
+    def __init__(self, tqdisable=True) -> None:
+        self.tqdisable = tqdisable
+
+    def _get_text(self, inst):
+        """Gets text from an article url (inst["link"])
+
+        Parameters
+        ----------
+        inst : dict
+            dictionary containing the link
+
+        Returns
+        -------
+        str
+            the article
+        """
+        try:
+            article = Article(
+                "http://"
+                + inst["link"]
+                .replace("http://", "")
+                .replace("https://", "")  # remove https that already exists
+            )
+            article.download()
+            article.parse()
+            text = article.text.strip().replace("\n", " ")
+            inst["text"] = text
+
+        except ArticleException:
+            inst["text"] = ""
+        self.pbar.update(1)
+        return inst["text"]
+
+    def get_articles(
+        self,
+        search_term,
+        start_time=(datetime.now() - timedelta(days=2)),
+        end_time=datetime.now(),
+    ):
+        """Gets articles for a single search term
+
+        Parameters
+        ----------
+        searchfor: str
+            term to search for
+        start_time : timedelta = (datetime.now() - timedelta(days=2))
+            how far back to search from
+        end_time : timedelta = datetime.now()
+            how recent to search from
+
+        Returns
+        -------
+        pandas.DataFrame
+            a dataframe of the results with columns ['title', 'author', 'source', 'desc',
+                                                    'text', 'datetime', 'url', 'urlToImage']
+        """
+
+        # use settimerange instead
+        results = finvizfinance(search_term).TickerNews()
+        results.rename(
+            columns={"Date": "datetime", "Link": "link", "Title": "title"}, inplace=True
+        )
+        self.pbar = tqdm(
+            total=len(results),
+            unit="article",
+            desc=search_term,
+            leave=False,
+            disable=self.tqdisable,
+        )
+        results = results.to_dict(orient="records")  # convert to dict
+
+        processes = []  # multi thread the execution
+        for i in results:
+            processes.append(
+                Thread(
+                    target=self._get_text,
+                    args=(i,),
+                )
+            )
+
+        # start
+        for proc in processes:
+            proc.start()
+        # join
+        for proc in processes:
+            proc.join()
+        self.pbar.close()
+        results = pd.DataFrame(results)
+        results.rename(columns={"link": "url"}, inplace=True)
+        return results
+
+
 class TwitterParser:
     """
     Gets tweets for analyzing
@@ -318,6 +434,8 @@ class TwitterParser:
     ----------
     bearer_token : str
         twitter api bearer token
+    tqdisable : bool
+        disale progressbars
 
     Methods
     -------
@@ -359,7 +477,7 @@ class TwitterParser:
         self,
         topic,
         pages=1,
-        start_time=(datetime.now() - timedelta(2)),
+        start_time=(datetime.now() - timedelta(days=2)),
         end_time=datetime.now(),
     ):  # how many days back to go
         """Get the tweets for a given topic
@@ -368,10 +486,10 @@ class TwitterParser:
         ----------
         topic : str
             topic to search for
-        start_time : str = (datetime.now() - timedelta(2))
-            how far back to search from in time format %Y-%m-%dT%H:%M:%SZ'
-        end_time : str = datetime.now()
-            how recent to search from in time format %Y-%m-%dT%H:%M:%SZ'
+        start_time : timedelta = (datetime.now() - timedelta(days=2))
+            how far back to search from
+        end_time : timedelta = datetime.now()
+            how recent to search from
 
         Returns
         -------
@@ -434,6 +552,8 @@ class TwintParser:
     ----------
     bearer_token : str
         twitter api bearer token
+    tqdisable : bool
+        disale progressbars
 
     Methods
     -------
@@ -478,7 +598,7 @@ class TwintParser:
         self,
         topic,
         pages=2,
-        start_time=(datetime.now() - timedelta(2)),
+        start_time=(datetime.now() - timedelta(days=2)),
         end_time=datetime.now(),
     ):  # how many days back to go
         """Get the tweets for a given topic
@@ -487,10 +607,10 @@ class TwintParser:
         ----------
         topic : str
             topic to search for
-        start_time : str = (datetime.now() - timedelta(2))
-            how far back to search from in time format %Y-%m-%dT%H:%M:%SZ'
+        start_time : str = (datetime.now() - timedelta(days=2))
+            how far back to search from
         end_time : str = datetime.now()
-            how recent to search from in time format %Y-%m-%dT%H:%M:%SZ'
+            how recent to search from
 
         Returns
         -------
@@ -517,6 +637,78 @@ class TwintParser:
         return tweets
 
 
+class TrendParser:
+    """
+    Interfaces with the google trends api
+
+    Attributes
+    ----------
+    trend : TrendReq
+        the trends object
+
+    Methods
+    -------
+        get_trends(topic)
+    """
+
+    def __init__(self, lang="en-US") -> None:
+        """
+        Parameters
+        ----------
+        lang : str = 'en-US'
+            language to search in
+        """
+
+        self.pytrends = TrendReq(hl=lang, tz=360)
+
+    def interest_over_time(
+        self,
+        topics_list,
+        start_time=(datetime.now() - timedelta(days=2)),
+        end_time=datetime.now(),
+        geo="",
+        cat=0,
+    ):
+        """Get the past trends for a topic
+
+        "Numbers represent search interest relative to the highest point on the
+        chart for the given region and time. A value of 100 is the peak
+        popularity for the term. A value of 50 means that the term is half as
+        popular. A score of 0 means there was not enough data for this term."
+
+        Parameters
+        ----------
+        topics_list : list(str)
+            list of topics to search for
+        start_time : timedelta = (datetime.now() - timedelta(days=2))
+            how far back to search from
+        end_time : timedelta = datetime.now()
+            how recent to search from
+        geo : str = ""
+            where in the world to search from (eg us, world, etc)
+        cat : int = 0
+            category to narrow results
+            (see https://github.com/pat310/google-trends-api/wiki/Google-Trends-Categories)
+
+        Returns
+        -------
+        results : pd.DataFrame
+            dataframe of results
+        """
+        try:
+            timeframe = (
+                start_time.strftime(TRENDS_TF) + " " + end_time.strftime(TRENDS_TF)
+            )
+            self.pytrends.build_payload(
+                topics_list, cat=cat, timeframe=timeframe, geo=geo, gprop=""
+            )
+            results = self.pytrends.interest_over_time()
+            return results
+        except Exception as e:
+            warnings.warn(f"Couldn't get trends ({e})")
+            return pd.DataFrame
+
+
 class Abraham:
     """
     Performs sentiment analysis on a search term by taking care of gathering
@@ -532,6 +724,12 @@ class Abraham:
         flair model for the tweets
     lemmatizer : WordNetLemmatizer
         lemmatizer from nltk
+    trendparser : TrendParser
+        trend parser
+    twitterparser
+        twitter parser
+    newparser
+        news parser
     news_source : str
         where to get the news from (google or newsapi)
     splitting : bool
@@ -555,8 +753,6 @@ class Abraham:
     _analyze_news_text(frame, section, recursive)
         takes a dataframe and a section and scores each row. if recursive=True, it will only
         analyze one sentence at time
-    get_articles
-        gets articles for a single search term
     twitter_sentiment
         takes a list of topics and gets the raw scores for each
         (per topic per text type per row)
@@ -579,6 +775,8 @@ class Abraham:
     summary_interval
         takes a list of topics and gets the NEWS+TWITTER AVG
         summary over each 'period' intervals from oldest to newest
+    interest_interval
+        gets the google trends ratio between a list of terms
     """
 
     def __init__(
@@ -612,7 +810,7 @@ class Abraham:
         if news_source == "newsapi":
             if newsapi_key:
                 self.news_source = "newsapi"
-                self.newsparser = NewsAPI(newsapi_key=newsapi_key)
+                self.newsparser = NewsAPIParser(newsapi_key=newsapi_key)
             else:
                 print(
                     "You requested newsapi but no key was provided. Defaulting to googlenews."
@@ -622,6 +820,7 @@ class Abraham:
         else:
             self.news_source = "google"
             self.newsparser = GoogleNewsParser()
+
         self.weights = weights
         self.loud = loud
 
@@ -630,6 +829,8 @@ class Abraham:
         else:
             self.twitterparser = TwintParser(tqdisable=True)
 
+        self.finvizparser = FinvizParser()
+        self.trendparser = TrendParser()
         # sentiment analysis directly
         self.vader = SentimentIntensityAnalyzer()
         self.lemmatizer = WordNetLemmatizer()
@@ -801,47 +1002,10 @@ class Abraham:
         newframe = newframe[newframe.sentiment != "NEUTRAL"]
         return newframe
 
-    def get_articles(
-        self,
-        topics: list,
-        start_time=(datetime.now() - timedelta(2)),
-        end_time=datetime.now(),
-    ) -> Dict:
-        """Takes a list of topics and returns a dict of topics : pd.dataframe
-
-        Parameters
-        ----------
-        topics : list
-            list of terms to search for
-        start_time : str = (datetime.now() - timedelta(2))
-            how far back to search from in time format %Y-%m-%dT%H:%M:%SZ'
-        end_time : str = datetime.now()
-            how recent to search from in time format %Y-%m-%dT%H:%M:%SZ'
-
-        Returns
-        -------
-        dict
-            in format {topic: <pd.DataFrame>, topic: <pd.DataFrame>, ... } with
-            dataframe being of the results with columns ['title', 'author',
-                'source', 'desc', 'text', 'datetime', 'url', 'urlToImage']
-            ex:
-            {
-                'coinbase': <pd.DataFrame>,
-                'bitcoin': <pd.DataFrame>,
-                ...
-            }
-        """
-        topic_results = {}
-        for topic in topics:
-            topic_results[topic] = self.newsparser.get_articles(
-                topic, start_time=start_time, end_time=end_time
-            )
-        return topic_results
-
     def news_summary(
         self,
         topics: list,
-        start_time=(datetime.now() - timedelta(2)),
+        start_time=(datetime.now() - timedelta(days=2)),
         end_time=datetime.now(),
     ):
         """Gets the summary sentiment for each topic
@@ -850,7 +1014,7 @@ class Abraham:
         ----------
         topics : list
             list of terms to search for
-        start_time : str = (datetime.now() - timedelta(2))
+        start_time : str = (datetime.now() - timedelta(days=2))
             how far back to search from in time format %Y-%m-%dT%H:%M:%SZ'
         end_time : str = datetime.now()
             how recent to search from in time format %Y-%m-%dT%H:%M:%SZ'
@@ -928,7 +1092,7 @@ class Abraham:
     def news_sentiment(
         self,
         topics: list,
-        start_time=(datetime.now() - timedelta(2)),
+        start_time=(datetime.now() - timedelta(days=2)),
         end_time=datetime.now(),
     ):
         """Gets the WHOLE sentiment for each topic. No or minimal averaging occurs.
@@ -937,7 +1101,7 @@ class Abraham:
         ----------
         topics : list
             list of terms to search for
-        start_time : timedelta = (datetime.now() - timedelta(2))
+        start_time : timedelta = (datetime.now() - timedelta(days=2))
             how far back to search from in time format %Y-%m-%dT%H:%M:%SZ'
         end_time : timedelta = datetime.now()
             how recent to search from in time format %Y-%m-%dT%H:%M:%SZ'
@@ -954,11 +1118,11 @@ class Abraham:
           0.173  0.827  0.000   -0.5859  Tesla working vehicle ...  2021-04-20T09:31:36Z
         """
 
-        articles = self.get_articles(
-            topics,
-            start_time=start_time,
-            end_time=end_time,
-        )
+        articles = {}
+        for topic in topics:
+            articles[topic] = self.newsparser.get_articles(
+                topic, start_time=start_time, end_time=end_time
+            )
         scores = {}
         for topic in articles:
             titles = self._analyze_flair_text(
@@ -977,7 +1141,7 @@ class Abraham:
         self,
         topics: list,
         size: int = 100,
-        start_time=(datetime.now() - timedelta(2)),
+        start_time=(datetime.now() - timedelta(days=2)),
         end_time=datetime.now(),
     ):
         """Gets the summary sentiment for each topic from twitter
@@ -986,7 +1150,7 @@ class Abraham:
         ----------
         topics : list
             list of terms to search for
-        start_time : timedelta = (datetime.now() - timedelta(2))
+        start_time : timedelta = (datetime.now() - timedelta(days=2))
             how far back to search from in time format %Y-%m-%dT%H:%M:%SZ'
         end_time : timedelta = datetime.now()
             how recent to search from in time format %Y-%m-%dT%H:%M:%SZ'
@@ -1032,7 +1196,7 @@ class Abraham:
         self,
         topics: list,
         size=100,
-        start_time=(datetime.now() - timedelta(2)),
+        start_time=(datetime.now() - timedelta(days=2)),
         end_time=datetime.now(),
     ):
         """Gets the WHOLE sentiment for each topic from twitter. No or minimal averaging occurs.
@@ -1043,7 +1207,7 @@ class Abraham:
             list of terms to search for
         size : int = 100
             roughly how many tweets to get
-        start_time : timedelta = (datetime.now() - timedelta(2))
+        start_time : timedelta = (datetime.now() - timedelta(days=2))
             how far back to search from in time format %Y-%m-%dT%H:%M:%SZ'
         end_time : timedelta = datetime.now()
             how recent to search from in time format %Y-%m-%dT%H:%M:%SZ'
@@ -1068,7 +1232,7 @@ class Abraham:
     def summary(
         self,
         topics: list,
-        start_time=(datetime.now() - timedelta(2)),
+        start_time=(datetime.now() - timedelta(days=2)),
         end_time=datetime.now(),
         weights={"news": 0.5, "twitter": 0.5},
     ):
@@ -1078,10 +1242,10 @@ class Abraham:
         ----------
         topics : list
             list of terms to search for
-        start_time : str = (datetime.now() - timedelta(2))
-            how far back to search from in time format %Y-%m-%dT%H:%M:%SZ'
-        end_time : str = datetime.now()
-            how recent to search from in time format %Y-%m-%dT%H:%M:%SZ'
+        start_time : timedelta = (datetime.now() - timedelta(days=2))
+            how far back to search from
+        end_time : timedelta = datetime.now()
+            how recent to search from
         weights : dict = {"news": 0.5, "twitter": 0.5}
             how to weight the news results to the twitter results
 
@@ -1354,4 +1518,36 @@ class Abraham:
                 # move back
                 now = pre
             results[topic] = df
-        return results
\ No newline at end of file
+        return results
+
+    def interest_interval(
+        self,
+        topics_list,
+        start_time=(datetime.now() - timedelta(days=365)),
+        end_time=datetime.now(),
+    ):
+        """Get the past trends for a topic
+
+        "Numbers represent search interest relative to the highest point on the
+        chart for the given region and time. A value of 100 is the peak
+        popularity for the term. A value of 50 means that the term is half as
+        popular. A score of 0 means there was not enough data for this term."
+
+        Parameters
+        ----------
+        topics_list : list(str)
+            list of topics to search for
+        start_time : timedelta = (datetime.now() - timedelta(days=365))
+            how far back to search from
+        end_time : timedelta = datetime.now()
+            how recent to search from
+
+        Returns
+        -------
+        results : pd.DataFrame
+            dataframe of results
+        """
+        results = self.trendparser.interest_over_time(
+            topics_list, start_time=start_time, end_time=end_time, geo="", cat=0
+        ).drop("isPartial", axis=1)
+        return results
